/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation.editingsupport;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.fortiss.tooling.base.annotation.AnnotationEntry;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;

/**
 * {@link EditingSupport} providing a simple text input field to display an manipulate
 * {@link IAnnotatedSpecification}s. It is used in all cases where no more specialized
 * {@link EditingSupport} is available.
 * 
 * @author diewald
 */
public class TextEditingSupport extends AnnotationEditingSupportBase {

	/** Text cell editor. */
	protected TextCellEditor cellEditor = null;

	/** Constructs a new {@link TextEditingSupport}. */
	public TextEditingSupport(ColumnViewer viewer,
			Class<? extends IAnnotatedSpecification> specClass) {
		super(viewer, specClass);

		cellEditor = new TextCellEditor((Composite)getViewer().getControl(), SWT.NONE);
	}

	/** {@inheritDoc} */
	@Override
	protected CellEditor getCellEditor(Object element) {
		return cellEditor;
	}

	/** {@inheritDoc} */
	@Override
	protected Object getValue(Object element) {
		// This method is triggered from several methods in EditingSupport, which - for this
		// TextEditingSupport - passes its result to TextCellEditor.doSetValue() which expects a
		// non-null String.
		Object rval = super.getValue(element);

		return (rval != null) ? rval.toString() : "";
	}

	/** {@inheritDoc} */
	@Override
	protected void doSetValue(AnnotationEntry annotationEntry, String value) throws Exception {
		super.doSetValue(annotationEntry, value.equals(UNSET_RESERVED_VALUE) ? null : value);
	}

	/** {@inheritDoc} */
	@Override
	protected void doSetValue(AnnotationEntry annotationEntry, Object value) throws Exception {
		if(value.toString().equals(UNSET_RESERVED_VALUE)) {
			value = null;
		}

		super.doSetValue(annotationEntry, value);
	}
}
