/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editor.fx.controller;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.ecore.EObject;

/**
 * An {@link KernelServiceBasedModelChangeProviderBase} for {@link EObject}s.
 * 
 * @author hoelzl
 */
final class EObjectModelChangeProvider extends KernelServiceBasedModelChangeProviderBase<EObject> {
	/** The model element. */
	private final EObject modelElement;
	/** The EMF notification listener. */
	private final EObjectElementAdapter notificationListener;

	/** Constructor. */
	public EObjectModelChangeProvider(EObject modelElement) {
		this.modelElement = modelElement;
		this.notificationListener = new EObjectElementAdapter();
	}

	/** {@inheritDoc} */
	@Override
	protected EObject getModelElement() {
		return modelElement;
	}

	/** {@inheritDoc} */
	@Override
	protected Adapter getNotificationListener() {
		return notificationListener;
	}

	/** Specialized adapter listening to model changes of {@link EObject}. */
	private class EObjectElementAdapter extends AdapterImpl {
		/** {@inheritDoc} */
		@Override
		public void notifyChanged(Notification event) {
			addNotification(event);
		}
	}
}
