/*-------------------------------------------------------------------------+
| Copyright 2020 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editor.fx.controller;

import static org.fortiss.tooling.base.ui.utils.ContextMenuUtils.createPrototypeMenu;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.fortiss.tooling.base.model.element.IModelElement;
import org.fortiss.tooling.common.ui.javafx.control.treetableview.DynamicTreeTableUIProviderBase;
import org.fortiss.tooling.kernel.ui.extension.base.FXEditorBase;
import org.fortiss.tooling.kernel.ui.service.IModelElementHandlerService;

import javafx.scene.Node;
import javafx.scene.control.ContextMenu;

/**
 * Base implementation for FX UIProvider for ModelElements.
 * 
 * @author bayha
 */
public abstract class ModelElementFXEditorUIProviderBase<T extends IModelElement>
		extends DynamicTreeTableUIProviderBase<T> {

	/** The editor used to post selection changes. */
	protected final FXEditorBase<?> editor;

	/** Constructor. */
	public ModelElementFXEditorUIProviderBase(FXEditorBase<?> editor) {
		this.editor = editor;
	}

	/** {@inheritDoc} */
	@Override
	public Node getIconNode(IModelElement element, int column) {
		if(element == null) {
			return null;
		}
		return IModelElementHandlerService.getInstance().getFXIcon(element);
	}

	/** {@inheritDoc} */
	@Override
	public ContextMenu createContextMenu(IModelElement element, int column) {
		if(element == null || column != 0) {
			return null;
		}
		ContextMenu contextMenu = new ContextMenu();
		contextMenu.getItems().addAll(createPrototypeMenu(element, null));
		return contextMenu;
	}

	/** {@inheritDoc} */
	@Override
	public void select(IModelElement oldValue, IModelElement newValue) {
		IStructuredSelection selection =
				newValue == null ? StructuredSelection.EMPTY : new StructuredSelection(newValue);
		editor.getSite().getSelectionProvider().setSelection(selection);
	}
}
