/*-------------------------------------------------------------------------+
| Copyright 2020 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editor.fx.visual;

import static javafx.scene.paint.Color.BLACK;
import static org.fortiss.tooling.base.layout.DefaultLayoutConstants.DEFAULT_CONNECTOR_SIZE;
import static org.fortiss.tooling.base.utils.LayoutDataUtils.getNodeBounds;

import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.model.layout.Rectangle;
import org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.IContentMVCBundle;
import org.fortiss.tooling.common.ui.javafx.lwfxef.visual.elliptic.EllipticContentVisualBase;

import javafx.geometry.Rectangle2D;
import javafx.scene.paint.Paint;

/**
 * {@link EllipticContentVisualBase} providing a default visual for ellipses in ellipses-and-wires
 * diagrams based on {@link ILayoutedModelElement}s.
 * 
 * @author munaro
 */
public abstract class LayoutedEllipticContentVisualBase<T extends ILayoutedModelElement>
		extends EllipticContentVisualBase {

	/** Constructor. */
	public LayoutedEllipticContentVisualBase(IContentMVCBundle mvcb, Class<T> modelType) {
		super(mvcb);

		// TODO(#3877): Move type checks to a common base class.
		Object model = mvcb.getModel();
		if(model == null) {
			throw new IllegalArgumentException("The given model is null!");
		}
		if(!modelType.isAssignableFrom(model.getClass())) {
			throw new IllegalArgumentException("Expected model of type " +
					modelType.getSimpleName() + ", but was " + model.getClass().getSimpleName());
		}
	}

	/** Returns the model element with the specified type. */
	// TODO(#3877): Move type checks to a common base class.
	@SuppressWarnings("unchecked")
	protected T getModelElement() {
		// Safe wild cast due to type check in constructor
		return (T)getModel();
	}

	/** {@inheritDoc} */
	@Override
	public Rectangle2D getModelBounds() {
		Rectangle r = getNodeBounds(getModelElement());
		double dc = DEFAULT_CONNECTOR_SIZE / 2 - 2;
		double dc2 = 2 * dc;
		return new Rectangle2D(r.getX() + dc, r.getY() + dc, r.getWidth() - dc2,
				r.getHeight() - dc2);
	}

	/** {@inheritDoc} */
	@Override
	protected Paint getBorderColor() {
		return BLACK;
	}

	/** {@inheritDoc} */
	@Override
	protected double getBorderWidth() {
		return 1;
	}

	/** {@inheritDoc} */
	@Override
	protected double getOpacity() {
		return 1.0;
	}

	/** {@inheritDoc} */
	@Override
	protected boolean requireSelectionForMoveGesture() {
		return false;
	}

	/** {@inheritDoc} */
	@Override
	protected boolean requireSelectionForResizeGesture() {
		return false;
	}
}
