/*******************************************************************************
 * Copyright (c) 2016, 2018 fortiss GmbH.
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v2.0 which is available at
 * http://www.eclipse.org/legal/epl-v20.html
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Florian Hoelzl (fortiss GmbH) - initial implementation
 *     Alexander Diewald (fortiss GmbH) - backport to AF3 & Integration
 *                                        improvements
 *
 *******************************************************************************/
package org.fortiss.tooling.common.ui.javafx.control.treetableview;

import java.util.Collection;
import java.util.HashMap;

import javafx.scene.control.TreeItem;

/**
 * {@link TreeItem} with support for dynamic children using the
 * {@link DynamicTreeItem#update()} method.
 */
public class DynamicTreeItem<T> extends TreeItem<T> {
	/** The viewer. */
	private DynamicTreeViewerBase<T> viewer;

	/** Constructor. */
	public DynamicTreeItem(T data, DynamicTreeViewerBase<T> viewer) {
		super(data);
		this.viewer = viewer;
	}

	/** Updates the children of this item from the underlying content model. */
	public void update() {
		// remember expanded state of children
		HashMap<T, TreeItem<T>> expanded = new HashMap<>();
		for(TreeItem<T> c : getChildren()) {
			if(c.isExpanded()) {
				expanded.put(c.getValue(), c);
			}
		}
		// get list of children and create tree items
		getChildren().clear();
		DynamicTreeContentProviderBase<T> cp = viewer.getContentProvider();
		for(T element : cp.getFilteredSortedChildren(getValue())) {
			DynamicTreeItem<T> dti;
			if(expanded.containsKey(element)) {
				dti = (DynamicTreeItem<T>)expanded.get(element);
			} else {
				dti = new DynamicTreeItem<T>(element, viewer);
			}
			getChildren().add(dti);
			dti.update();
		}
	}

	/** {@inheritDoc} */
	@Override
	public boolean isLeaf() {
		Collection<? extends Object> children = viewer.getContentProvider().getChildren(getValue());
		return children == null || children.isEmpty();
	}
}
