/*******************************************************************************
 * Copyright (c) 2019, 2020 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Apache License, Version 2.0 which is available at
 * https://www.apache.org/licenses/LICENSE-2.0.
 *
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/
package org.fortiss.tooling.common.ui.javafx.lwfxef.controller.base;

import static org.fortiss.tooling.common.ui.javafx.lwfxef.FeedbackChange.locationFeedbackChange;

import org.fortiss.tooling.common.ui.javafx.lwfxef.DiagramViewerFeatures;
import org.fortiss.tooling.common.ui.javafx.lwfxef.FeedbackChange;

/** Utility methods used for computation of locations of mouse dragging operations. */
/* package */ final class DraggingUtils {
	/** Clamps the given location and snaps it to the closest grid point. */
	public static FeedbackChange clampAndSnapMoveLocationToGridPoint(double x, double y, double dx,
			double dy, DiagramViewerFeatures features) {
		return clampAndSnapMoveLocation(x, y, dx, dy, features, false);
	}

	/** Clamps the given location and snaps it to the closest center between four grid points. */
	public static FeedbackChange clampAndSnapMoveLocationToGridCenterPoint(double x, double y,
			double dx, double dy, DiagramViewerFeatures features) {
		return clampAndSnapMoveLocation(x, y, dx, dy, features, true);
	}

	/** Clamps the given location and snaps it to the grid (possibly centered). */
	private static FeedbackChange clampAndSnapMoveLocation(double x, double y, double dx, double dy,
			DiagramViewerFeatures features, boolean snapToCenter) {
		// add 10 to compensate the mouse cursor hit point
		// and make the anchorage stick closer to the cursor
		double newx = x + dx + 10;
		double newy = y + dy + 10;
		// clamp
		if(features.isDrawOuterBorder()) {
			// clamp one space from origin
			newx = Math.max(newx, features.getHorizontalSpacing());
			newy = Math.max(newy, features.getVerticalSpacing());
		} else {
			// clamp negativ coordinates
			newx = Math.max(newx, 0);
			newy = Math.max(newy, 0);
		}
		// snap to grid
		if(features.useSnapToGrid()) {
			newx = snapToCenter ? features.snapToCenterX(newx) : features.snapToGridX(newx);
			newy = snapToCenter ? features.snapToCenterY(newy) : features.snapToGridY(newy);
		}
		dx = newx - x;
		dy = newy - y;
		return locationFeedbackChange(dx, dy);
	}
}
