/*******************************************************************************
 * Copyright (c) 2019, 2020 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Apache License, Version 2.0 which is available at
 * https://www.apache.org/licenses/LICENSE-2.0.
 *
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/
package org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.impl;

import org.fortiss.tooling.common.ui.javafx.lwfxef.DiagramViewer;
import org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.IAnchorageMVCBundle;
import org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.IDiagramMVCBundle;
import org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.ILinkMVCBundle;
import org.fortiss.tooling.common.ui.javafx.lwfxef.visual.ILinkVisual;

/**
 * {@link MVCBundleBase} for connections providing access to both end-point {@link MVCBundleBase}s.
 */
public class LinkMVCBundle extends MVCBundleBase implements ILinkMVCBundle {
	/** The {@link IDiagramMVCBundle}, which is the parent of all links. */
	private final IDiagramMVCBundle parent;
	/** The start-point {@link IAnchorageMVCBundle}. */
	private IAnchorageMVCBundle start;
	/** The end-point {@link IAnchorageMVCBundle}. */
	private IAnchorageMVCBundle end;

	/** Constructor. */
	public LinkMVCBundle(Object m, IDiagramMVCBundle parent, IAnchorageMVCBundle start,
			IAnchorageMVCBundle end, DiagramViewer v) {
		super(m, v);
		this.parent = parent;
		parent.getLinks().add(this);
		this.start = start;
		start.getOutgoingLinks().add(this);
		this.end = end;
		end.getIncomingLinks().add(this);
	}

	/** {@inheritDoc} */
	@Override
	public final void remove() {
		parent.getLinks().remove(this);
		start.getOutgoingLinks().remove(this);
		end.getIncomingLinks().remove(this);
	}

	/** {@inheritDoc} */
	@Override
	public IAnchorageMVCBundle getStartAnchorage() {
		return start;
	}

	/** Sets the start bundle. */
	public void setStartAnchorage(IAnchorageMVCBundle start) {
		this.start = start;
	}

	/** {@inheritDoc} */
	@Override
	public IAnchorageMVCBundle getEndAnchorage() {
		return end;
	}

	/** Sets the end bundle. */
	public void setEndAnchorage(IAnchorageMVCBundle end) {
		this.end = end;
	}

	/** {@inheritDoc} */
	@Override
	public IDiagramMVCBundle getDiagramMVCBundle() {
		return parent;
	}

	/** {@inheritDoc} */
	@Override
	public ILinkVisual getVisual() {
		return (ILinkVisual)super.getVisual();
	}

	/** {@inheritDoc} */
	@Override
	public final void updateLinkBundle(IAnchorageMVCBundle newStart, IAnchorageMVCBundle newEnd) {
		if(start != null && start != newStart) {
			start.getOutgoingLinks().remove(this);
			start = newStart;
			start.getOutgoingLinks().add(this);
		}
		if(end != null && end != newEnd) {
			end.getIncomingLinks().remove(this);
			end = newEnd;
			end.getIncomingLinks().add(this);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void changed(Object element) {
		getViewer().updateVisual(this);
	}
}
