/*******************************************************************************
 * Copyright (c) 2019, 2020 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Apache License, Version 2.0 which is available at
 * https://www.apache.org/licenses/LICENSE-2.0.
 *
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/
package org.fortiss.tooling.common.ui.javafx.lwfxef.visual;

import org.fortiss.tooling.common.ui.javafx.lwfxef.DiagramCoordinate;

import javafx.scene.Node;

/**
 * {@link IVisual} for content nodes moving freely on the diagram background.
 * Content nodes can normally be resized and have anchorage visuals.
 */
public interface IContentVisual extends IVisual {
	/**
	 * Returns the location on the border of this content visual for the given
	 * {@link IContentAnchorageVisual}.
	 */
	DiagramCoordinate getAnchorageLocation(IContentAnchorageVisual visual);

	/** Returns whether the given node is the expand/collapse widget. */
	boolean isExpandCollapseWidget(Node node);

	/**
	 * Returns whether anchorage visuals attached to this visual should be updated when this content
	 * visual is hovered over by the mouse pointer. The default is {@code false}.
	 * 
	 * @see IContentAnchorageVisual#updateSiblingAnchorageVisualsOnHover()
	 */
	boolean updateAttachedAnchorageVisualsOnHover();
}
