/*******************************************************************************
 * Copyright (c) 2019, 2020 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Apache License, Version 2.0 which is available at
 * https://www.apache.org/licenses/LICENSE-2.0.
 *
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/
package org.fortiss.tooling.common.ui.javafx.lwfxef.visual.widgets;

import static javafx.scene.paint.Color.BLACK;
import static javafx.scene.paint.Color.TRANSPARENT;

import javafx.collections.ObservableList;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.paint.Paint;
import javafx.scene.shape.Line;
import javafx.scene.shape.Rectangle;

/** Widget showing a plus / minus indicator used for hierarchical content nodes. */
public final class ExpandCollapseWidget extends Group {
	/** The rectangle of the indicator. */
	private final Rectangle borderRect = new Rectangle();
	/** The horizontal line used for both plus and minus state. */
	private final Line horizontalLine = new Line();
	/** The vertical line used for plus state. */
	private final Line verticalLine = new Line();

	/** Constructor. */
	public ExpandCollapseWidget(double x, double y, double w, double h, double inset) {
		update(x, y, w, h, inset);

		ObservableList<Node> c = getChildren();
		c.add(borderRect);
		c.add(horizontalLine);

		setColor(BLACK, TRANSPARENT);
	}

	/** Updates the location, size and insets of the widget. */
	public void update(double x, double y, double w, double h, double inset) {
		borderRect.setX(x);
		borderRect.setY(y);
		borderRect.setWidth(w);
		borderRect.setHeight(h);

		horizontalLine.setStartX(x + inset);
		horizontalLine.setEndX(x + w - inset);
		horizontalLine.setStartY(y + h / 2);
		horizontalLine.setEndY(y + h / 2);

		verticalLine.setStartX(x + w / 2);
		verticalLine.setEndX(x + w / 2);
		verticalLine.setStartY(y + inset);
		verticalLine.setEndY(y + h - inset);
	}

	/** Sets the state of the widget. */
	public void setState(boolean expanded) {
		if(expanded) {
			getChildren().remove(verticalLine);
		} else if(verticalLine.getParent() != this) {
			// avoid duplicate add
			getChildren().add(verticalLine);
		}
	}

	/** Sets the fill of the rectangle and the stroke color. */
	public void setColor(Paint stroke, Paint fill) {
		borderRect.setFill(fill);
		borderRect.setStroke(stroke);
		verticalLine.setStroke(stroke);
		horizontalLine.setStroke(stroke);
	}

	/** Determines if the given node is this widget or a part of it. */
	public boolean isHit(Node hit) {
		return hit == this || hit.getParent() == this;
	}
}
