/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.extension.base;

import org.fortiss.tooling.kernel.extension.data.LogMessage;
import org.fortiss.tooling.kernel.service.ILoggingService;

/**
 * {@link LogMessage} that presents a defined string in a dialog to the user. It uses the
 * {@link ILoggingService} to emit messages from non-UI plugins.
 * 
 * @author diewald
 */
public class DialogMessage extends LogMessage {

	/** Denotes the type of message for presenting the correct dialog type. */
	@SuppressWarnings("javadoc")
	public enum MSG_TYPE {
	INFORMATION, WARNING, ERROR
	}

	/** Holds the severity of the message. */
	private MSG_TYPE severity = MSG_TYPE.INFORMATION;

	/** Constructor. Default severity: {@link MSG_TYPE#INFORMATION}. */
	public DialogMessage(String dialogMsg) {
		super(dialogMsg);
	}

	/** Constructor. Allows defining the severity. */
	public DialogMessage(String dialogMsg, MSG_TYPE severity) {
		super(dialogMsg);
		this.severity = severity;
	}

	/** Returns the severity of the message. */
	public MSG_TYPE getSeverity() {
		return severity;
	}
}
