/*-------------------------------------------------------------------------+
| Copyright 2011 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.extension.exception;

import java.util.List;

import org.fortiss.tooling.kernel.extension.ITransformationProvider;
import org.fortiss.tooling.kernel.extension.data.ITransformationContext;
import org.fortiss.tooling.kernel.extension.data.TransformationProviderChain;

/**
 * An exception thrown by
 * {@link TransformationProviderChain#transform(Object, ITransformationContext)} if the
 * transformation failed at some point of the chain. The failed
 * transformation provider and the successful intermediate results can be
 * obtained from this exception.
 * 
 * @author hoelzl
 */
public class TransformationFailedException extends Exception {

	/** Stores the failed provider. */
	private final ITransformationProvider failedProvider;

	/** Stores the list of intermediate results. */
	private final List<Object> successfulResults;

	/** Constructor. */
	public TransformationFailedException(ITransformationProvider failedProvider,
			List<Object> successfulResults, TransformationFailedException cause) {
		super(cause);
		this.failedProvider = failedProvider;
		this.successfulResults = successfulResults;
	}

	/** Constructor. */
	public TransformationFailedException(String message, ITransformationProvider failedProvider,
			List<Object> successfulResults, TransformationFailedException cause) {
		super(message, cause);
		this.failedProvider = failedProvider;
		this.successfulResults = successfulResults;
	}

	/** Returns the failed provider. */
	public ITransformationProvider getFailedProvider() {
		return failedProvider;
	}

	/** Returns the list of successful intermediate results. */
	public List<Object> getSuccessfulResults() {
		return successfulResults;
	}
}
