/*-------------------------------------------------------------------------+
| Copyright 2020 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.service;

import org.fortiss.tooling.kernel.extension.ICommandLineSwitchHandler;
import org.fortiss.tooling.kernel.internal.CommandLineInterfaceService;

/**
 * Service to register handlers for command line switches and their arguments. The handlers are
 * executed after the startup of all other service to ensure the presence of the full feature set
 * when this service is executed.
 * 
 * @author diewald
 */
public interface ICommandLineInterfaceService {

	/** Returns the internal implementation of the {@link ICommandLineInterfaceService}. */
	public static ICommandLineInterfaceService getInstance() {
		return CommandLineInterfaceService.getInstance();
	}

	/**
	 * Executes the registered {@link ICommandLineSwitchHandler}s for the switches that have been
	 * passed to the program executable. This method can be executed only once during the
	 * application lifecycle, otherwise it degenerates to a no-op.
	 */
	void executeHandlers();

	/**
	 * Registers an {@link ICommandLineSwitchHandler} that is executed if the given switch has
	 * been passed to the program executable.
	 */
	void registerHandler(String cliSwitch, ICommandLineSwitchHandler cliHandler);
}
