/*******************************************************************************
 * Copyright (c) 2017, 2018 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Apache License, Version 2.0, which is available at 
 * https://www.apache.org/licenses/LICENSE-2.0.
 *
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/
package org.fortiss.tooling.spiderchart.control;

import org.fortiss.tooling.spiderchart.model.SpiderChart;
import org.fortiss.tooling.spiderchart.style.ChartStyle;

import javafx.geometry.Rectangle2D;

/**
 * Base class for elements of the spider chart.
 * 
 * @author hoelzl
 */
public abstract class SpiderChartControlBase {
	/** The spider chart this component belongs to. */
	protected final SpiderChart chart;
	/** The spider chart style information. */
	protected final ChartStyle style;
	/** The height of the component. */
	protected double height;
	/** The width of the component. */
	protected double width;
	/** The x location. */
	protected double x;
	/** The y location. */
	protected double y;

	/** Constructor. */
	public SpiderChartControlBase(SpiderChart chart, ChartStyle style) {
		this.chart = chart;
		this.style = style;
	}

	/** Sets the bounding rectangle area of this widget. */
	public void setBounds(double x, double y, double width, double height) {
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
	}

	/** Returns x. */
	public final double getX() {
		return x;
	}

	/** Returns y. */
	public final double getY() {
		return y;
	}

	/** Returns width. */
	public final double getWidth() {
		return width;
	}

	/** Returns height. */
	public final double getHeight() {
		return height;
	}

	/** Returns the bounding rectangle. */
	public final Rectangle2D getBounds() {
		return new Rectangle2D(x, y, width, height);
	}
}
