/*******************************************************************************
 * Copyright (c) 2017, 2018 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Apache License, Version 2.0, which is available at 
 * https://www.apache.org/licenses/LICENSE-2.0.
 *
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/
package org.fortiss.tooling.spiderchart.control;

import org.fortiss.tooling.common.ui.javafx.style.FontStyle;
import org.fortiss.tooling.spiderchart.model.SpiderChart;
import org.fortiss.tooling.spiderchart.style.ChartStyle;

import javafx.geometry.Dimension2D;
import javafx.scene.canvas.GraphicsContext;

/**
 * Class used for drawing the spider chart title.
 * 
 * @author hoelzl
 */
public final class SpiderChartTitleControl extends SpiderChartControlBase {
	/** Constructor */
	public SpiderChartTitleControl(SpiderChart chart, ChartStyle style) {
		super(chart, style);
	}

	/** Used to draw the title */
	public void draw(GraphicsContext gc) {
		if(!style.isShowTitle()) {
			return;
		}

		String text = chart.getTitle();
		if(text == null || text.trim().length() == 0) {
			return;
		}

		gc.save();
		FontStyle fontStyle = style.getTitleStyle();
		Dimension2D extent = fontStyle.getTextBounds(text);
		double x = getX() + getWidth() / 2 - extent.getWidth() / 2;
		double y = getY() + getHeight() / 2 - extent.getHeight() / 2;
		fontStyle.drawText(gc, text, x, y);
		gc.restore();
	}
}
