/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package test.org.fortiss.tooling.spiderchart;

import static javafx.scene.paint.Color.BLUE;
import static javafx.scene.paint.Color.DARKGRAY;
import static javafx.scene.paint.Color.GREEN;
import static javafx.scene.paint.Color.LIGHTGRAY;
import static javafx.scene.paint.Color.RED;
import static org.fortiss.tooling.common.ui.javafx.style.FontStyle.BLACK_VERDANA_10PT;
import static org.fortiss.tooling.common.ui.javafx.style.FontStyle.BLACK_VERDANA_12PT;
import static org.fortiss.tooling.common.ui.javafx.style.FontStyle.BLACK_VERDANA_14PT;
import static org.fortiss.tooling.common.ui.javafx.style.FontStyle.BLACK_VERDANA_8PT;
import static org.fortiss.tooling.common.ui.javafx.style.LineStyle.SOLID_BLACK_1PT;

import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.Comparator;

import org.fortiss.tooling.common.ui.javafx.layout.CompositeFXControllerBase;
import org.fortiss.tooling.common.ui.javafx.style.FillStyle;
import org.fortiss.tooling.common.ui.javafx.style.FontStyle;
import org.fortiss.tooling.common.ui.javafx.style.LineStyle;
import org.fortiss.tooling.spiderchart.control.SpiderChartViewer;
import org.fortiss.tooling.spiderchart.model.DataSeries;
import org.fortiss.tooling.spiderchart.model.DoubleAxis;
import org.fortiss.tooling.spiderchart.model.EnumerationAxis;
import org.fortiss.tooling.spiderchart.model.SpiderChart;
import org.fortiss.tooling.spiderchart.style.AxisStyle;
import org.fortiss.tooling.spiderchart.style.ChartStyle;
import org.fortiss.tooling.spiderchart.style.DataSeriesStyle;
import org.fortiss.tooling.spiderchart.style.LegendStyle;

import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.Pane;

/**
 * Test view for the {@link SpiderChartViewer}.
 *
 * @author hoelzl
 */
@SuppressWarnings("unchecked")
public class SpiderChartExampleFXController extends CompositeFXControllerBase<AnchorPane, Node> {

	/** Pane containing the spider chart's canvas. */
	@FXML
	Pane pane;

	/** Anchor pane on which the spider chart is drawn. */
	@FXML
	Pane anchorPane;

	/** Constructs the spider chart viewer. */
	private static SpiderChartViewer buildSpiderChart() {
		SpiderChart spiderChart = new SpiderChart();
		spiderChart.setTitle("Smartphone Comparison Scale");
		spiderChart.setLegendLabel("Legend");

		DoubleAxis battery = new DoubleAxis("Battery", 0.0, 4.0);
		DoubleAxis camera = new DoubleAxis("Camera", 0.0, 7.0);
		DoubleAxis display = new DoubleAxis("Display", 0.0, 5.0);
		DoubleAxis memory = new DoubleAxis("Memory", 0.0, 5.0);
		DoubleAxis brand = new DoubleAxis("Brand", 0.0, 5.0);

		Screen sc0 = new Screen(0); // Unused dummy element for spider chart center point
		Screen sc100 = new Screen(100);
		Screen sc240 = new Screen(240);
		Screen sc320 = new Screen(320);
		Screen sc640 = new Screen(640);
		Screen sc1024 = new Screen(1024);
		Screen[] screens = new Screen[] {sc0, sc100, sc240, sc320, sc640, sc1024};
		EnumerationAxis<Screen> screen = new EnumerationAxis<Screen>("Screen Width",
				screenComparator, Arrays.asList(screens));

		spiderChart.addAxis(battery);
		spiderChart.addAxis(camera);
		spiderChart.addAxis(display);
		spiderChart.addAxis(memory);
		spiderChart.addAxis(brand);
		spiderChart.addAxis(screen);

		DataSeries iPhoneData = new DataSeries("Phone 1");
		iPhoneData.setPoint(battery, 4.0);
		iPhoneData.setPoint(camera, 3.5);
		iPhoneData.setPoint(display, 4.0);
		iPhoneData.setPoint(memory, 4.6);
		iPhoneData.setPoint(brand, 5.0);
		iPhoneData.setPoint(screen, sc240);
		spiderChart.addData(iPhoneData);

		DataSeries nexusData = new DataSeries("Phone 2");
		nexusData.setPoint(battery, 3.0);
		nexusData.setPoint(camera, 4.5);
		nexusData.setPoint(display, 3.0);
		nexusData.setPoint(memory, 3.6);
		nexusData.setPoint(brand, 4.0);
		nexusData.setPoint(screen, sc640);
		spiderChart.addData(nexusData);

		ChartStyle chartStyle = new ChartStyle(true, true, true);
		chartStyle.setUseIndividualAxisSegments(false);
		chartStyle.setTitleStyle(new FontStyle("Verdana", 14, BLUE.brighter()));

		LegendStyle legendStyle = new LegendStyle(false, 5, BLACK_VERDANA_12PT);
		chartStyle.setLegendStyle(legendStyle);

		AxisStyle aStyle3Segs = new AxisStyle(SOLID_BLACK_1PT, BLACK_VERDANA_14PT, 3,
				BLACK_VERDANA_8PT, new DecimalFormat("#.##"));
		chartStyle.setAxisStyle(battery, aStyle3Segs);
		AxisStyle aStyle6Segs = new AxisStyle(SOLID_BLACK_1PT, BLACK_VERDANA_14PT, 6,
				BLACK_VERDANA_8PT, new DecimalFormat("#.##"));
		chartStyle.setAxisStyle(camera, aStyle6Segs);
		AxisStyle aStyle4Segs = new AxisStyle(SOLID_BLACK_1PT, BLACK_VERDANA_14PT, 4,
				BLACK_VERDANA_8PT, new DecimalFormat("#.##"));
		chartStyle.setAxisStyle(display, aStyle4Segs);
		chartStyle.setAxisStyle(memory, aStyle4Segs);
		chartStyle.setAxisStyle(brand, aStyle4Segs);
		chartStyle.setAxisStyle(screen, aStyle6Segs);
		chartStyle.setAxisSegments(4);

		chartStyle.setBackgroundFillStyle(new FillStyle(LIGHTGRAY));
		chartStyle.setBackgroundLineStyle(new LineStyle(DARKGRAY));

		LineStyle olive1pt = new LineStyle(GREEN);
		FillStyle oliveFill = new FillStyle(GREEN, 0.25);
		DataSeriesStyle iphoneStyle = new DataSeriesStyle(olive1pt, oliveFill, true, true,
				BLACK_VERDANA_10PT, 7, new DecimalFormat("#.#"));
		chartStyle.setDataSeriesStyle(iPhoneData, iphoneStyle);

		LineStyle navy1pt = new LineStyle(RED);
		FillStyle navyFill = new FillStyle(RED, 0.25);
		DataSeriesStyle nexusStyle = new DataSeriesStyle(navy1pt, navyFill, true, true,
				BLACK_VERDANA_10PT, 7, new DecimalFormat("#.#"));
		chartStyle.setDataSeriesStyle(nexusData, nexusStyle);

		SpiderChartViewer viewer = new SpiderChartViewer(spiderChart, chartStyle);
		return viewer;
	}

	/** Data class used to demonstrate enumeration data axes. */
	private static class Screen implements Comparable<Screen> {
		/** Screen width in pixels. */
		public int width = 0;

		/** Constructor. */
		public Screen(int width) {
			this.width = width;
		}

		/** {@inheritDoc} */
		@Override
		public int compareTo(Screen o) {
			return screenComparator.compare(this, o);
		}

		/** {@inheritDoc} */
		@Override
		public String toString() {
			return width + "px";
		}
	}

	/** {@link Comparator} implementation for {@link Screen}s . */
	private static Comparator<Screen> screenComparator = new Comparator<Screen>() {
		@Override
		public int compare(Screen o1, Screen o2) {
			return o1.width - o2.width;
		}
	};

	/** {@inheritDoc} */
	@Override
	public String getFXMLLocation() {
		return "SpiderChartExample.fxml";
	}

	/** {@inheritDoc} */
	@Override
	public void initialize() {
		Pane viewerPane = buildSpiderChart().getViewerPane();
		viewerPane.setPrefSize(500, 500);
		anchorPane.getChildren().add(viewerPane);
	}
}
